package com.parnes.telldus.client;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.RootPanel;

/**
 * Entry point classes define <code>onModuleLoad()</code>.
 */
public class Telldus implements EntryPoint {
	
	Button onButton0, offButton0, onButton1, offButton1, onButton2, offButton2;

	/**
	 * Create a remote service proxy to talk to the server-side Greeting service.
	 */
	private final TelldusServiceAsync powerService = GWT
			.create(TelldusService.class);

	/**
	 * This is the entry point method.
	 */
	public void onModuleLoad() {
		onButton0 = new Button("On 1");
		offButton0 = new Button("Off 1");
		onButton1 = new Button("On 2");
		offButton1 = new Button("Off 2");
		onButton2 = new Button("On 3");
		offButton2 = new Button("Off 3");

		// We can add style names to widgets
		onButton0.addStyleName("Button");
		offButton0.addStyleName("Button");
		onButton1.addStyleName("Button");
		offButton1.addStyleName("Button");
		onButton2.addStyleName("Button");
		offButton2.addStyleName("Button");

		// Add the nameField and sendButton to the RootPanel
		// Use RootPanel.get() to get the entire body element
		RootPanel.get("onButtonContainer0").add(onButton0);
		RootPanel.get("offButtonContainer0").add(offButton0);
		RootPanel.get("onButtonContainer1").add(onButton1);
		RootPanel.get("offButtonContainer1").add(offButton1);
		RootPanel.get("onButtonContainer2").add(onButton2);
		RootPanel.get("offButtonContainer2").add(offButton2);
		
		// Focus the cursor on the on-button when the app loads
		onButton0.setFocus(true);
		
		final Label statusTextLabel = new Label();
		RootPanel.get("statusTextContainer").add(statusTextLabel);

		// Create a handler for the sendButton and nameField
		class MyHandler implements ClickHandler {
			
			/**
			 * Fired when the user clicks on the sendButton.
			 */
			public void onClick(ClickEvent event) {
				sendCommandToServer(event);
			}

			/**
			 * Send the name from the nameField to the server and wait for a response.
			 */
			private void sendCommandToServer(ClickEvent event) {
				Button button = (Button)event.getSource();
				boolean on = false;
				int unit = 0; 
				if(button == onButton0) {
					on = true;
					unit = 0; 
				} else if(button == offButton0) {
					on = false;
					unit = 0; 
				} else if(button == onButton1) {
					on = true;
					unit = 1; 
				} else if(button == offButton1) {
					on = false;
					unit = 1; 
				} else if(button == onButton2) {
					on = true;
					unit = 2; 
				} else if(button == offButton2) {
					on = false;
					unit = 2; 
				};
				
				statusTextLabel.setText("Sending command: " + unit + "," + on);
				toggleAllButtons(false);
				powerService.power(unit, on, 
						new AsyncCallback<Boolean>() {
							public void onFailure(Throwable caught) {
								String statusText = "Power control: Error communicating with the server.";
								setStatusText(statusText);
								toggleAllButtons(true);
							}

							public void onSuccess(Boolean result) {
								String statusText = "Power control: " + (result?"OK":"ERROR");
								setStatusText(statusText);
								toggleAllButtons(true);
							}
							
							void setStatusText(String text) {
								statusTextLabel.setText(text);
							}

						});
			}
		}

		// Add a handler to send the name to the server
		MyHandler handler = new MyHandler();
		onButton0.addClickHandler(handler);
		offButton0.addClickHandler(handler);
		onButton1.addClickHandler(handler);
		offButton1.addClickHandler(handler);
		onButton2.addClickHandler(handler);
		offButton2.addClickHandler(handler);
	}
	
	void toggleAllButtons(boolean enabled)  {
		onButton0.setEnabled(enabled);
		offButton0.setEnabled(enabled);
		onButton1.setEnabled(enabled);
		offButton1.setEnabled(enabled);
		onButton2.setEnabled(enabled);
		offButton2.setEnabled(enabled);
	}
}
